@@ -19,7 +19,7 @@
  * <http://www.gnu.org/licenses/>.
  */
 
-#include <QtTest>
+#include <QTest>
 #include <QFileInfo>
 
 #include <kpluginloader.h>
@@ -199,7 +199,7 @@ private Q_SLOTS:
         KPluginLoader aplugin(QStringLiteral("alwaysunloadplugin"));
         QJsonObject ametadata = aplugin.metaData();
         QVERIFY(!ametadata.isEmpty());
-        QVERIFY(ametadata.keys().contains(QStringLiteral("IID")));
+        QVERIFY(ametadata.keys().contains(QLatin1String("IID")));
         QJsonValue ametadata_metadata = ametadata.value(QStringLiteral("MetaData"));
         QVERIFY(ametadata_metadata.toObject().isEmpty());
         QVERIFY(!aplugin.isLoaded()); // didn't load anything
@@ -261,19 +261,19 @@ private Q_SLOTS:
 
         //try filter
         plugins = KPluginLoader::instantiatePlugins(temp.path(), [](const KPluginMetaData & md) {
-            return md.pluginId() == QStringLiteral("jsonplugin");
+            return md.pluginId() == QLatin1String("jsonplugin");
         });
         QCOMPARE(plugins.size(), 1);
         QCOMPARE(plugins[0]->metaObject()->className(), "jsonpluginfa");
         qDeleteAll(plugins);
 
         plugins = KPluginLoader::instantiatePlugins(temp.path(), [](const KPluginMetaData & md) {
-            return md.pluginId() == QStringLiteral("unversionedplugin");
+            return md.pluginId() == QLatin1String("unversionedplugin");
         });
         QCOMPARE(plugins.size(), 0);
 
         plugins = KPluginLoader::instantiatePlugins(temp.path(), [](const KPluginMetaData & md) {
-            return md.pluginId() == QStringLiteral("foobar"); // ID does not match file name, is set in JSON
+            return md.pluginId() == QLatin1String("foobar"); // ID does not match file name, is set in JSON
         });
         QCOMPARE(plugins.size(), 1);
         QCOMPARE(plugins[0]->metaObject()->className(), "jsonplugin2");
@@ -305,24 +305,18 @@ private Q_SLOTS:
 
     void testFindPlugins()
     {
-        const QString plugin1Path = KPluginLoader::findPlugin(QStringLiteral("jsonplugin"));
-        QVERIFY2(!plugin1Path.isEmpty(), qPrintable(plugin1Path));
-        const QString plugin2Path = KPluginLoader::findPlugin(QStringLiteral("unversionedplugin"));
-        QVERIFY2(!plugin2Path.isEmpty(), qPrintable(plugin2Path));
-        const QString plugin3Path = KPluginLoader::findPlugin(QStringLiteral("jsonplugin2"));
-        QVERIFY2(!plugin3Path.isEmpty(), qPrintable(plugin3Path));
-
         QTemporaryDir temp;
         QVERIFY(temp.isValid());
         QDir dir(temp.path());
         QVERIFY(dir.mkdir(QStringLiteral("kpluginmetadatatest")));
         QVERIFY(dir.cd(QStringLiteral("kpluginmetadatatest")));
-        QVERIFY2(QFile::copy(plugin1Path, dir.absoluteFilePath(QFileInfo(plugin1Path).fileName())),
-                 qPrintable(dir.absoluteFilePath(QFileInfo(plugin1Path).fileName())));
-        QVERIFY2(QFile::copy(plugin2Path, dir.absoluteFilePath(QFileInfo(plugin2Path).fileName())),
-                 qPrintable(dir.absoluteFilePath(QFileInfo(plugin2Path).fileName())));
-        QVERIFY2(QFile::copy(plugin3Path, dir.absoluteFilePath(QFileInfo(plugin3Path).fileName())),
-                 qPrintable(dir.absoluteFilePath(QFileInfo(plugin3Path).fileName())));
+        for (const QString &name : { QStringLiteral("jsonplugin"), QStringLiteral("unversionedplugin"), QStringLiteral("jsonplugin2") }) {
+            const QString pluginPath = KPluginLoader::findPlugin(name);
+            QVERIFY2(!pluginPath.isEmpty(), qPrintable(pluginPath));
+            QVERIFY2(QFile::copy(pluginPath, dir.absoluteFilePath(QFileInfo(pluginPath).fileName())),
+                    qPrintable(dir.absoluteFilePath(QFileInfo(pluginPath).fileName())));
+        }
+
         LibraryPathRestorer restorer(QCoreApplication::libraryPaths());
         // we only want plugins from our temporary dir
         QCoreApplication::setLibraryPaths(QStringList() << temp.path());
@@ -351,6 +345,20 @@ private Q_SLOTS:
         QCOMPARE(plugins[0].description(), QStringLiteral("This is another plugin"));
         QCOMPARE(plugins[1].description(), QStringLiteral("This is a plugin"));
 
+        // mimetype filter. Only one match, jsonplugin2 is specific to text/html.
+        auto supportTextPlain = [](const KPluginMetaData &metaData) { return metaData.supportsMimeType(QLatin1String("text/plain")); };
+        plugins = KPluginLoader::findPlugins(QStringLiteral("kpluginmetadatatest"), supportTextPlain);
+        QCOMPARE(plugins.size(), 1);
+        QCOMPARE(plugins[0].description(), QStringLiteral("This is a plugin"));
+
+        // mimetype filter. Two matches, both support text/html, via inheritance.
+        auto supportTextHtml = [](const KPluginMetaData &metaData) { return metaData.supportsMimeType(QLatin1String("text/html")); };
+        plugins = KPluginLoader::findPlugins(QStringLiteral("kpluginmetadatatest"), supportTextHtml);
+        std::sort(plugins.begin(), plugins.end(), sortPlugins);
+        QCOMPARE(plugins.size(), 2);
+        QCOMPARE(plugins[0].description(), QStringLiteral("This is another plugin"));
+        QCOMPARE(plugins[1].description(), QStringLiteral("This is a plugin"));
+
         // invalid std::function as filter
         plugins = KPluginLoader::findPlugins(QStringLiteral("kpluginmetadatatest"));
         std::sort(plugins.begin(), plugins.end(), sortPlugins);
