@@ -22,7 +22,7 @@
 #include "../src/lib/text/ktexttohtml.h"
 #include "../src/lib/text/ktexttohtml_p.h"
 
-#include <QtTest>
+#include <QTest>
 #include <QDebug>
 #include <QUrl>
 
@@ -159,8 +159,8 @@ void KTextToHTMLTest::testGetUrl2(const QString &left, const QString &right)
     urls << QStringLiteral("user:pass@www.kde.org:1234/  \n  sub/path:with:colon/  \n\t   \t   special(123)?") +
             QStringLiteral("\n\t  \n\t   a=1#anchor[bla]");
 
-    foreach (const QString &schema, schemas) {
-        foreach (QString url, urls) {   //krazy:exclude=foreach
+    for (const QString &schema : qAsConst(schemas)) {
+        for (QString url : qAsConst(urls)) {   //krazy:exclude=foreach
             // by definition: if the URL is enclosed in brackets, the URL itself is not allowed
             // to contain the closing bracket, as this would be detected as the end of the URL
             if ((left.length() == 1) && (url.contains(right[ 0 ]))) {
@@ -207,8 +207,8 @@ void KTextToHTMLTest::testGetUrl2(const QString &left, const QString &right)
     QStringList starts;
     starts << QStringLiteral("www") << QStringLiteral("ftp") << QStringLiteral("news:www");
 
-    foreach (const QString &start, starts) {
-        foreach (QString url, urlsWithoutSchema) {   //krazy:exclude=foreach
+    for (const QString &start : qAsConst(starts)) {
+        for (QString url : qAsConst(urlsWithoutSchema)) {
             // by definition: if the URL is enclosed in brackets, the URL itself is not allowed
             // to contain the closing bracket, as this would be detected as the end of the URL
             if ((left.length() == 1) && (url.contains(right[ 0 ]))) {
@@ -335,6 +335,39 @@ void KTextToHTMLTest::testHtmlConvert_data()
     QTest::newRow("") << "*Ouais !* *Yes!*"
                       << KTextToHTML::Options(KTextToHTML::PreserveSpaces | KTextToHTML::HighlightText)
                       << "<b>*Ouais !*</b> <b>*Yes!*</b>";
+
+    QTest::newRow("multispace") << "*Ouais     foo*"
+                      << KTextToHTML::Options(KTextToHTML::PreserveSpaces | KTextToHTML::HighlightText)
+                      << "<b>*Ouais     foo*</b>";
+
+    QTest::newRow("multispace3") << "*Ouais:     foo*"
+                      << KTextToHTML::Options(KTextToHTML::PreserveSpaces | KTextToHTML::HighlightText)
+                      << "<b>*Ouais:     foo*</b>";
+
+    QTest::newRow("multi-") << "** Ouais:  foo **"
+                      << KTextToHTML::Options(KTextToHTML::PreserveSpaces | KTextToHTML::HighlightText)
+                      << "** Ouais:&nbsp;&nbsp;foo **";
+
+    QTest::newRow("multi-") << "*** Ouais:  foo ***"
+                      << KTextToHTML::Options(KTextToHTML::PreserveSpaces | KTextToHTML::HighlightText)
+                      << "*** Ouais:&nbsp;&nbsp;foo ***";
+
+    QTest::newRow("nohtmlversion") << "* Ouais:     foo *"
+                      << KTextToHTML::Options(KTextToHTML::PreserveSpaces | KTextToHTML::HighlightText)
+                      << "* Ouais:&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;foo *";
+
+    QTest::newRow("nohtmlversion2") << "*Ouais:     foo *"
+                      << KTextToHTML::Options(KTextToHTML::PreserveSpaces | KTextToHTML::HighlightText)
+                      << "*Ouais:&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;foo *";
+
+    QTest::newRow("nohtmlversion3") << "* Ouais:     foo*"
+                      << KTextToHTML::Options(KTextToHTML::PreserveSpaces | KTextToHTML::HighlightText)
+                      << "* Ouais:&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;foo*";
+
+    QTest::newRow("nohtmlversion3") << "* Ouais: *ff sfsdf* foo *"
+                      << KTextToHTML::Options(KTextToHTML::PreserveSpaces | KTextToHTML::HighlightText)
+                      << "* Ouais: <b>*ff sfsdf*</b> foo *";
+
     QTest::newRow("") << "the /etc/{rsyslog.d,syslog-ng.d}/package.rpmnew file"
                       << KTextToHTML::Options(KTextToHTML::PreserveSpaces | KTextToHTML::HighlightText)
                       << "the /etc/{rsyslog.d,syslog-ng.d}/package.rpmnew file";
@@ -492,6 +525,10 @@ void KTextToHTMLTest::testHtmlConvert_data()
     QTest::newRow("tel extraction disabled") << "call +49123456789 now"
         << KTextToHTML::Options(KTextToHTML::PreserveSpaces)
         << "call +49123456789 now";
+
+    QTest::newRow("bug-414360") << "https://www.openstreetmap.org/directions?engine=graphhopper_foot&route=44.85765%2C-0.55931%3B44.85713%2C-0.56117#map=18/44.85756/-0.56094"
+                     << KTextToHTML::Options(KTextToHTML::PreserveSpaces)
+                     << "<a href=\"https://www.openstreetmap.org/directions?engine=graphhopper_foot&route=44.85765%2C-0.55931%3B44.85713%2C-0.56117#map=18/44.85756/-0.56094\">https://www.openstreetmap.org/directions?engine=graphhopper_foot&amp;route=44.85765%2C-0.55931%3B44.85713%2C-0.56117#map=18/44.85756/-0.56094</a>";
 }
 
 
@@ -504,7 +541,7 @@ void KTextToHTMLTest::testHtmlConvert()
     QEXPECT_FAIL("punctation-bug", "Linklocator does not properly detect punctation as boundaries",
                  Continue);
 
-    QString actualHtml = KTextToHTML::convertToHtml(plainText, flags);
+    const QString actualHtml = KTextToHTML::convertToHtml(plainText, flags);
     QCOMPARE(actualHtml, htmlText);
 }
 
