@@ -23,13 +23,15 @@
 #define KJOB_H
 
 #include <kcoreaddons_export.h>
-#include <QtCore/QObject>
-#include <QtCore/QPair>
+#include <QObject>
+#include <QPair>
 
 class KJobUiDelegate;
 
 class KJobPrivate;
 /**
+ * @class KJob kjob.h KJob
+ *
  * The base class for all jobs.
  * For all jobs created in an application, the code looks like
  *
@@ -75,7 +77,7 @@ class KJobPrivate;
  * reimplemented by any subclasses that introduce new
  * error codes.
  *
- * @note: KJob and its subclasses are meant to be used
+ * @note KJob and its subclasses are meant to be used
  * in a fire-and-forget way. Jobs will delete themselves
  * when they finish using deleteLater() (although this
  * behaviour can be changed), so a job instance will
@@ -84,34 +86,36 @@ class KJobPrivate;
 class KCOREADDONS_EXPORT KJob : public QObject
 {
     Q_OBJECT
-    Q_ENUMS(KillVerbosity Capability Unit)
-    Q_FLAGS(Capabilities)
     Q_PROPERTY(int error READ error NOTIFY result)
     Q_PROPERTY(QString errorText READ errorText NOTIFY result)
     Q_PROPERTY(QString errorString READ errorString NOTIFY result)
     Q_PROPERTY(unsigned long percent READ percent NOTIFY percent)
+    Q_PROPERTY(Capabilities capabilities READ capabilities CONSTANT)
 
 public:
     enum Unit { Bytes, Files, Directories };
+    Q_ENUM(Unit)
 
     enum Capability { NoCapabilities = 0x0000,
                       Killable       = 0x0001,
                       Suspendable    = 0x0002
                     };
+    Q_ENUM(Capability)
 
     Q_DECLARE_FLAGS(Capabilities, Capability)
+    Q_FLAG(Capabilities)
 
     /**
      * Creates a new KJob object.
      *
      * @param parent the parent QObject
      */
-    explicit KJob(QObject *parent = 0);
+    explicit KJob(QObject *parent = nullptr);
 
     /**
      * Destroys a KJob object.
      */
-    virtual ~KJob();
+    ~KJob() override;
 
     /**
      * Attach a UI delegate to this job.
@@ -127,7 +131,7 @@ public:
     /**
      * Retrieves the delegate attached to this job.
      *
-     * @return the delegate attached to this job, or 0 if there's no such delegate
+     * @return the delegate attached to this job, or @c nullptr if there's no such delegate
      */
     KJobUiDelegate *uiDelegate() const;
 
@@ -171,6 +175,7 @@ public:
     Q_SCRIPTABLE virtual void start() = 0;
 
     enum KillVerbosity { Quietly, EmitResult };
+    Q_ENUM(KillVerbosity)
 
 public Q_SLOTS:
     /**
@@ -248,9 +253,9 @@ public:
      * called, which usually wreaks havoc.
      *
      * Note that the event loop started by this method does not process user input events, which means
-     * your user interface will effectivly be blocked. Other events like paint or network events are
+     * your user interface will effectively be blocked. Other events like paint or network events are
      * still being processed. The advantage of not processing user input events is that the chance of
-     * accidental reentrancy is greatly reduced. Still you should avoid calling this function.
+     * accidental reentrance is greatly reduced. Still you should avoid calling this function.
      *
      * @return true if the job has been executed without error, false otherwise
      */
@@ -368,7 +373,7 @@ Q_SIGNALS:
      * @see result
      */
     void finished(KJob *job
-#if !defined(DOXYGEN_SHOULD_SKIP_THIS)
+#if !defined(K_DOXYGEN)
                   , QPrivateSignal
 #endif
                  );
@@ -382,7 +387,7 @@ Q_SIGNALS:
      * @param job the job that emitted this signal
      */
     void suspended(KJob *job
-#if !defined(DOXYGEN_SHOULD_SKIP_THIS)
+#if !defined(K_DOXYGEN)
                    , QPrivateSignal
 #endif
                   );
@@ -396,7 +401,7 @@ Q_SIGNALS:
      * @param job the job that emitted this signal
      */
     void resumed(KJob *job
-#if !defined(DOXYGEN_SHOULD_SKIP_THIS)
+#if !defined(K_DOXYGEN)
                  , QPrivateSignal
 #endif
                 );
@@ -416,7 +421,7 @@ Q_SIGNALS:
      * @see kill
      */
     void result(KJob *job
-#if !defined(DOXYGEN_SHOULD_SKIP_THIS)
+#if !defined(K_DOXYGEN)
                 , QPrivateSignal
 #endif
                );
@@ -435,9 +440,8 @@ Q_SIGNALS:
      * @param field2 second field (localized name and value)
      */
     void description(KJob *job, const QString &title,
-                     const QPair<QString, QString> &field1 = qMakePair(QString(), QString()),
-                     const QPair<QString, QString> &field2 = qMakePair(QString(), QString()));
-
+                     const QPair<QString, QString> &field1 = QPair<QString, QString>(),
+                     const QPair<QString, QString> &field2 = QPair<QString, QString>());
     /**
      * Emitted to display state information about this job.
      * Examples of message are "Resolving host", "Connecting to host...", etc.
@@ -458,15 +462,14 @@ Q_SIGNALS:
     void warning(KJob *job, const QString &plain, const QString &rich = QString());
 
 Q_SIGNALS:
-#if !defined(Q_MOC_RUN) && !defined(DOXYGEN_SHOULD_SKIP_THIS) && !defined(IN_IDE_PARSER)
-private: // don't tell moc, doxygen or kdevelop, but those signals are in fact private
-#endif
+    // These signals must be connected from KIO::KCoreDirLister (among others),
+    // therefore they must be public.
     /**
      * Emitted when we know the amount the job will have to process. The unit of this
      * amount is sent too. It can be emitted several times if the job manages several
      * different units.
      *
-     * This is a private signal, it can't be emitted directly by subclasses of
+     * @note This is a private signal, it shouldn't be emitted directly by subclasses of
      * KJob, use setTotalAmount() instead.
      *
      * @param job the job that emitted this signal
@@ -480,7 +483,7 @@ private: // don't tell moc, doxygen or kdevelop, but those signals are in fact p
      * The unit of this amount is sent too. It can be emitted several times if the job
      * manages several different units.
      *
-     * This is a private signal, it can't be emitted directly by subclasses of
+     * @note This is a private signal, it shouldn't be emitted directly by subclasses of
      * KJob, use setProcessedAmount() instead.
      *
      * @param job the job that emitted this signal
@@ -493,7 +496,7 @@ private: // don't tell moc, doxygen or kdevelop, but those signals are in fact p
      * Emitted when we know the size of this job (data size in bytes for transfers,
      * number of entries for listings, etc).
      *
-     * This is a private signal, it can't be emitted directly by subclasses of
+     * @note This is a private signal, it shouldn't be emitted directly by subclasses of
      * KJob, use setTotalAmount() instead.
      *
      * @param job the job that emitted this signal
@@ -505,7 +508,7 @@ private: // don't tell moc, doxygen or kdevelop, but those signals are in fact p
      * Regularly emitted to show the progress of this job
      * (current data size in bytes for transfers, entries listed, etc.).
      *
-     * This is a private signal, it can't be emitted directly by subclasses of
+     * @note This is a private signal, it shouldn't be emitted directly by subclasses of
      * KJob, use setProcessedAmount() instead.
      *
      * @param job the job that emitted this signal
@@ -519,7 +522,7 @@ private: // don't tell moc, doxygen or kdevelop, but those signals are in fact p
      * a progress bar very easily. (see KProgressBar).
      * Note that this signal is not emitted for finished jobs.
      *
-     * This is a private signal, it can't be emitted directly by subclasses of
+     * @note This is a private signal, it shouldn't be emitted directly by subclasses of
      * KJob, use emitPercent(), setPercent() setTotalAmount() or
      * setProcessedAmount() instead.
      *
@@ -531,7 +534,7 @@ private: // don't tell moc, doxygen or kdevelop, but those signals are in fact p
     /**
      * Emitted to display information about the speed of this job.
      *
-     * This is a private signal, it can't be emitted directly by subclasses of
+     * @note This is a private signal, it shouldn't be emitted directly by subclasses of
      * KJob, use emitSpeed() instead.
      *
      * @param job the job that emitted this signal
@@ -609,7 +612,7 @@ protected:
      * It first notifies the observers to hide the progress for this job using
      * the finished() signal.
      *
-     * @note: Deletes this job using deleteLater().
+     * @note Deletes this job using deleteLater().
      *
      * @see result()
      * @see finished()
@@ -640,6 +643,8 @@ protected:
     KJob(KJobPrivate &dd, QObject *parent);
 
 private:
+    void finishJob(bool emitResult);
+
     Q_PRIVATE_SLOT(d_func(), void _k_speedTimeout())
     Q_DECLARE_PRIVATE(KJob)
 };
