@@ -30,8 +30,8 @@
 #include <QString>
 #include <QSharedDataPointer>
 #include <QVariant>
+#include <qcontainerfwd.h>
 
-template <class T> class QList;
 class QCommandLineParser;
 class QJsonObject;
 class KAboutData;
@@ -469,12 +469,18 @@ public:
      */
     static KAboutData *pluginData(const QString &componentName);
 
+#if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 65)
     /**
      * Creates a @c KAboutData from the given @p plugin metadata
      *
      * @since 5.18
+     * @deprecated Since 5.65, use KAboutPluginDialog to show info about a plugin
+     * instead of KAboutApplicationDialog, with the latter having had been the
+     * only known need for this conversion.
      */
+    KCOREADDONS_DEPRECATED_VERSION(5, 65, "See API docs")
     static KAboutData fromPluginMetaData(const KPluginMetaData &plugin);
+#endif
 
 public:
     /**
@@ -744,8 +750,11 @@ public:
      * Use this if you need to have an application logo
      * in AboutData other than the application icon.
      *
-     * Because KAboutData is a core class it cannot use QImage directly,
-     * so this is a QVariant that should contain a QImage.
+     * Because KAboutData is a core class it cannot use QImage/QPixmap/QIcon directly,
+     * so this is a QVariant that should contain a QImage/QPixmap/QIcon.
+     *
+     * QIcon should be preferred, to be able to properly handle HiDPI scaling.
+     * If a QIcon is provided, it will be used at a typical size of 48x48.
      *
      * @param image logo image.
      * @see programLogo()
@@ -958,8 +967,8 @@ public:
     /**
      * Returns the program logo image.
      *
-     * Because KAboutData is a core class it cannot use QImage directly,
-     * so this is a QVariant containing a QImage.
+     * Because KAboutData is a core class it cannot use QImage/QPixmap/QIcon directly,
+     * so this is a QVariant containing a QImage/QPixmap/QIcon.
      *
      * @return the program logo data, or a null image if there is
      *         no custom application logo defined.
