@@ -37,6 +37,146 @@ KFormatPrivate::~KFormatPrivate()
 {
 }
 
+constexpr double bpow(int exp) {
+    return (exp > 0) ? 2.0 * bpow(exp - 1) :
+        (exp < 0) ? 0.5 * bpow(exp + 1) :
+        1.0;
+}
+
+QString KFormatPrivate::formatValue(double value,
+                        KFormat::Unit unit,
+                        QString unitString,
+                        int precision,
+                        KFormat::UnitPrefix prefix,
+                        KFormat::BinaryUnitDialect dialect) const
+{
+    if (dialect <= KFormat::DefaultBinaryDialect || dialect > KFormat::LastBinaryDialect) {
+        dialect = KFormat::IECBinaryDialect;
+    }
+
+    if (static_cast<int>(prefix) < static_cast<int>(KFormat::UnitPrefix::Yocto) ||
+        static_cast<int>(prefix) > static_cast<int>(KFormat::UnitPrefix::Yotta)) {
+        prefix = KFormat::UnitPrefix::AutoAdjust;
+    }
+
+    double multiplier = 1024.0;
+    if (dialect == KFormat::MetricBinaryDialect) {
+        multiplier = 1000.0;
+    }
+
+    int power = 0;
+    if (prefix == KFormat::UnitPrefix::AutoAdjust) {
+        double adjustValue = qAbs(value);
+        while (adjustValue >= multiplier) {
+            adjustValue /= multiplier;
+            power += 1;
+        }
+        while (adjustValue && adjustValue < 1.0) {
+            adjustValue *= multiplier;
+            power -= 1;
+        }
+        const KFormat::UnitPrefix map[] = {
+            KFormat::UnitPrefix::Yocto, // -8
+            KFormat::UnitPrefix::Zepto,
+            KFormat::UnitPrefix::Atto,
+            KFormat::UnitPrefix::Femto,
+            KFormat::UnitPrefix::Pico,
+            KFormat::UnitPrefix::Nano,
+            KFormat::UnitPrefix::Micro,
+            KFormat::UnitPrefix::Milli,
+            KFormat::UnitPrefix::Unity, // 0
+            KFormat::UnitPrefix::Kilo,
+            KFormat::UnitPrefix::Mega,
+            KFormat::UnitPrefix::Giga,
+            KFormat::UnitPrefix::Tera,
+            KFormat::UnitPrefix::Peta,
+            KFormat::UnitPrefix::Exa,
+            KFormat::UnitPrefix::Zetta,
+            KFormat::UnitPrefix::Yotta, // 8
+        };
+        power = std::max(-8, std::min(8, power));
+        prefix = map[power + 8];
+    }
+
+    if (prefix == KFormat::UnitPrefix::Unity &&
+        unit == KFormat::Unit::Byte) {
+        precision = 0;
+    }
+
+    struct PrefixMapEntry
+    {
+        KFormat::UnitPrefix prefix;
+        double decimalFactor;
+        double binaryFactor;
+        QString prefixCharSI;
+        QString prefixCharIEC;
+    };
+
+    const PrefixMapEntry map[] = {
+        { KFormat::UnitPrefix::Yocto, 1e-24, bpow(-80), tr("y", "SI prefix for 10^⁻24"), QString() },
+        { KFormat::UnitPrefix::Zepto, 1e-21, bpow(-70), tr("z", "SI prefix for 10^⁻21"), QString() },
+        { KFormat::UnitPrefix::Atto,  1e-18, bpow(-60), tr("a", "SI prefix for 10^⁻18"), QString() },
+        { KFormat::UnitPrefix::Femto, 1e-15, bpow(-50), tr("f", "SI prefix for 10^⁻15"), QString() },
+        { KFormat::UnitPrefix::Pico,  1e-12, bpow(-40), tr("p", "SI prefix for 10^⁻12"), QString() },
+        { KFormat::UnitPrefix::Nano,  1e-9,  bpow(-30), tr("n", "SI prefix for 10^⁻9") , QString() },
+        { KFormat::UnitPrefix::Micro, 1e-6,  bpow(-20), tr("µ", "SI prefix for 10^⁻6") , QString() },
+        { KFormat::UnitPrefix::Milli, 1e-3,  bpow(-10), tr("m", "SI prefix for 10^⁻3") , QString() },
+        { KFormat::UnitPrefix::Unity, 1.0,   1.0      , QString()                      , QString() },
+        { KFormat::UnitPrefix::Kilo,  1e3,   bpow(10) , tr("k", "SI prefix for 10^3")  , tr("Ki", "IEC binary prefix for 2^10") },
+        { KFormat::UnitPrefix::Mega,  1e6,   bpow(20) , tr("M", "SI prefix for 10^6")  , tr("Mi", "IEC binary prefix for 2^20") },
+        { KFormat::UnitPrefix::Giga,  1e9,   bpow(30) , tr("G", "SI prefix for 10^9")  , tr("Gi", "IEC binary prefix for 2^30") },
+        { KFormat::UnitPrefix::Tera,  1e12,  bpow(40) , tr("T", "SI prefix for 10^12") , tr("Ti", "IEC binary prefix for 2^40") },
+        { KFormat::UnitPrefix::Peta,  1e15,  bpow(50) , tr("P", "SI prefix for 10^15") , tr("Pi", "IEC binary prefix for 2^50") },
+        { KFormat::UnitPrefix::Exa,   1e18,  bpow(60) , tr("E", "SI prefix for 10^18") , tr("Ei", "IEC binary prefix for 2^60") },
+        { KFormat::UnitPrefix::Zetta, 1e21,  bpow(70) , tr("Z", "SI prefix for 10^21") , tr("Zi", "IEC binary prefix for 2^70") },
+        { KFormat::UnitPrefix::Yotta, 1e24,  bpow(80) , tr("Y", "SI prefix for 10^24") , tr("Yi", "IEC binary prefix for 2^80") },
+    };
+
+    auto entry = std::find_if(std::begin(map), std::end(map),
+            [prefix](const PrefixMapEntry& e) { return e.prefix == prefix; });
+
+    switch (unit) {
+        case KFormat::Unit::Bit:
+            unitString = tr("bit", "Symbol of binary digit");
+            break;
+        case KFormat::Unit::Byte:
+            unitString = tr("B", "Symbol of byte");
+            break;
+        case KFormat::Unit::Meter:
+            unitString = tr("m", "Symbol of meter");
+            break;
+        case KFormat::Unit::Hertz:
+            unitString = tr("Hz", "Symbol of hertz");
+            break;
+        case KFormat::Unit::Other:
+            break;
+    }
+
+    if (prefix == KFormat::UnitPrefix::Unity) {
+        QString numString = m_locale.toString(value, 'f', precision);
+        //: value without prefix, format "<val> <unit>"
+        return tr("%1 %2", "no Prefix").arg(numString, unitString);
+    }
+
+    QString prefixString;
+    if (dialect == KFormat::MetricBinaryDialect) {
+        value /= entry->decimalFactor;
+        prefixString = entry->prefixCharSI;
+    } else {
+        value /= entry->binaryFactor;
+        if (dialect == KFormat::IECBinaryDialect) {
+            prefixString = entry->prefixCharIEC;
+        } else {
+            prefixString = entry->prefixCharSI.toUpper();
+        }
+    }
+
+    QString numString = m_locale.toString(value, 'f', precision);
+
+    //: value with prefix, format "<val> <prefix><unit>"
+    return tr("%1 %2%3", "MetricBinaryDialect").arg(numString, prefixString, unitString);
+}
+
 QString KFormatPrivate::formatByteSize(double size, int precision,
                                        KFormat::BinaryUnitDialect dialect, KFormat::BinarySizeUnits units) const
 {
@@ -190,10 +330,10 @@ QString KFormatPrivate::formatDuration(quint64 msecs, KFormat::DurationFormatOpt
     quint64 ms = msecs;
     if (options & KFormat::HideSeconds) {
         //round to nearest minute
-        ms = qRound(ms / (qreal)MSecsInMinute) * MSecsInMinute ;
+        ms = qRound64(ms / (qreal)MSecsInMinute) * MSecsInMinute ;
     } else if (!(options & KFormat::ShowMilliseconds)) {
         //round to nearest second
-        ms = qRound(ms / (qreal)MSecsInSecond) * MSecsInSecond ;
+        ms = qRound64(ms / (qreal)MSecsInSecond) * MSecsInSecond ;
     }
 
     int hours = ms / MSecsInHour;
@@ -343,22 +483,19 @@ QString KFormatPrivate::formatSpelloutDuration(quint64 msecs) const
     if (days && hours) {
         /*: @item:intext days and hours. This uses the previous item:intext messages.
             If this does not fit the grammar of your language please contact the i18n team to solve the problem */
-        return tr("%1 and %2").arg(formatSingleDuration(Days, days))
-               .arg(formatSingleDuration(Hours, hours));
+        return tr("%1 and %2").arg(formatSingleDuration(Days, days), formatSingleDuration(Hours, hours));
     } else if (days) {
         return formatSingleDuration(Days, days);
     } else if (hours && minutes) {
         /*: @item:intext hours and minutes. This uses the previous item:intext messages.
             If this does not fit the grammar of your language please contact the i18n team to solve the problem */
-        return tr("%1 and %2").arg(formatSingleDuration(Hours, hours))
-               .arg(formatSingleDuration(Minutes, minutes));
+        return tr("%1 and %2").arg(formatSingleDuration(Hours, hours), formatSingleDuration(Minutes, minutes));
     } else if (hours) {
         return formatSingleDuration(Hours, hours);
     } else if (minutes && seconds) {
         /*: @item:intext minutes and seconds. This uses the previous item:intext messages.
             If this does not fit the grammar of your language please contact the i18n team to solve the problem */
-        return tr("%1 and %2").arg(formatSingleDuration(Minutes, minutes))
-               .arg(formatSingleDuration(Seconds, seconds));
+        return tr("%1 and %2").arg(formatSingleDuration(Minutes, minutes), formatSingleDuration(Seconds, seconds));
     } else if (minutes) {
         return formatSingleDuration(Minutes, minutes);
     } else {
@@ -372,67 +509,34 @@ QString KFormatPrivate::formatRelativeDate(const QDate &date, QLocale::FormatTyp
         return tr("Invalid date", "used when a relative date string can't be generated because the date is invalid");
     }
 
-    const int daysTo = QDate::currentDate().daysTo(date);
-    if (daysTo > 7 || daysTo < -7) {
+    const qint64 daysTo = QDate::currentDate().daysTo(date);
+    if (daysTo > 2 || daysTo < -2) {
         return m_locale.toString(date, format);
     }
 
     switch (daysTo) {
+    case 2:
+        return tr("In two days");
     case 1:
         return tr("Tomorrow");
     case 0:
         return tr("Today");
     case -1:
         return tr("Yesterday");
-    }
-
-    if (daysTo < -1) {
-        switch (date.dayOfWeek()) {
-        case 1:
-            return tr("Last Monday",    "most recent such day before today");
-        case 2:
-            return tr("Last Tuesday",   "most recent such day before today");
-        case 3:
-            return tr("Last Wednesday", "most recent such day before today");
-        case 4:
-            return tr("Last Thursday",  "most recent such day before today");
-        case 5:
-            return tr("Last Friday",    "most recent such day before today");
-        case 6:
-            return tr("Last Saturday",  "most recent such day before today");
-        case 7:
-            return tr("Last Sunday",    "most recent such day before today");
-        }
-    } else if (daysTo > 1) {
-        switch (date.dayOfWeek()) {
-        case 1:
-            return tr("Next Monday",    "the next such day after today");
-        case 2:
-            return tr("Next Tuesday",   "the next such day after today");
-        case 3:
-            return tr("Next Wednesday", "the next such day after today");
-        case 4:
-            return tr("Next Thursday",  "the next such day after today");
-        case 5:
-            return tr("Next Friday",    "the next such day after today");
-        case 6:
-            return tr("Next Saturday",  "the next such day after today");
-        case 7:
-            return tr("Next Sunday",    "the next such day after today");
-        }
+    case -2:
+        return tr("Two days ago");
     }
     Q_UNREACHABLE();
 }
 
 QString KFormatPrivate::formatRelativeDateTime(const QDateTime &dateTime, QLocale::FormatType format) const
 {
-    int daysTo = QDate::currentDate().daysTo(dateTime.date());
-    if (daysTo > 7 || daysTo < -7) {
+    const qint64 daysTo = QDate::currentDate().daysTo(dateTime.date());
+    if (daysTo > 2 || daysTo < -2) {
         return m_locale.toString(dateTime, format);
     }
 
     /*: relative datetime with %1 result of formatReleativeDate() and %2 the formatted time
         If this does not fit the grammar of your language please contact the i18n team to solve the problem */
-    return tr("%1, %2").arg(formatRelativeDate(dateTime.date(), format))
-           .arg(m_locale.toString(dateTime.time(), format));
+    return tr("%1, %2").arg(formatRelativeDate(dateTime.date(), format), m_locale.toString(dateTime.time(), format));
 }
